@@ -110,7 +110,7 @@ public:
         {
             for (int i = 0; i < numInps; i++)
             {
-                String inpfile = _tf(basename + ".input_" + (i + '0') + ".npy");
+                String inpfile = _tf(basename + cv::format(".input_%d.npy", i));
                 inps.push_back(blobFromNPY(inpfile));
             }
         }
@@ -124,7 +124,7 @@ public:
         {
             for (int i = 0; i < numOuts; i++)
             {
-                String outfile = _tf(basename + "_" + (i + '0') + ".npy");
+                String outfile = _tf(basename + cv::format("_%d.npy", i));
                 refs.push_back(blobFromNPY(outfile));
             }
         }
@@ -146,7 +146,7 @@ public:
         {
             for (int i = 0; i < numInps; i++)
             {
-                net.setInput(inps[i], inp_name + "_" + (i + '0'));
+                net.setInput(inps[i], inp_name + cv::format("_%d", i));
             }
         }
         else
@@ -169,17 +169,8 @@ TEST_P(Test_Caffe_layers, Softmax)
 
 TEST_P(Test_Caffe_layers, LRN)
 {
-    double l1 = 0.0, lInf = 0.0;
-    // The OpenCL kernels use the native_ math functions which have
-    // implementation defined accuracy, so we use relaxed thresholds. See
-    // https://github.com/opencv/opencv/issues/9821 for more details.
-    if (target == DNN_TARGET_OPENCL)
-    {
-        l1 = 0.01;
-        lInf = 0.01;
-    }
-    testLayerUsingCaffeModels("layer_lrn_spatial", false, true, l1, lInf);
-    testLayerUsingCaffeModels("layer_lrn_channels", false, true, l1, lInf);
+    testLayerUsingCaffeModels("layer_lrn_spatial");
+    testLayerUsingCaffeModels("layer_lrn_channels");
 }
 
 TEST_P(Test_Caffe_layers, Convolution)
@@ -189,28 +180,20 @@ TEST_P(Test_Caffe_layers, Convolution)
 
 TEST_P(Test_Caffe_layers, DeConvolution)
 {
+    if(target == DNN_TARGET_CUDA_FP16)
+        applyTestTag(CV_TEST_TAG_DNN_SKIP_CUDA_FP16);
     testLayerUsingCaffeModels("layer_deconvolution", true, false);
 }
 
 TEST_P(Test_Caffe_layers, InnerProduct)
 {
-#if defined(INF_ENGINE_RELEASE) && INF_ENGINE_VER_MAJOR_LT(2021040000)
     if (backend == DNN_BACKEND_INFERENCE_ENGINE_NN_BUILDER_2019)
         applyTestTag(CV_TEST_TAG_DNN_SKIP_IE_NN_BUILDER);
     if (backend == DNN_BACKEND_INFERENCE_ENGINE_NGRAPH)
         applyTestTag(CV_TEST_TAG_DNN_SKIP_IE_NGRAPH);
-#endif
-#if defined(INF_ENGINE_RELEASE) && INF_ENGINE_VER_MAJOR_EQ(2021040000)
-    // IE exception: Ngraph operation Reshape with name Reshape_4219609 has dynamic output shape on 0 port, but CPU plug-in supports only static shape
-    if (backend == DNN_BACKEND_INFERENCE_ENGINE_NGRAPH && (target == DNN_TARGET_OPENCL || target == DNN_TARGET_OPENCL_FP16))
-        applyTestTag(target == DNN_TARGET_OPENCL ? CV_TEST_TAG_DNN_SKIP_IE_OPENCL : CV_TEST_TAG_DNN_SKIP_IE_OPENCL_FP16,
-            CV_TEST_TAG_DNN_SKIP_IE_NGRAPH, CV_TEST_TAG_DNN_SKIP_IE_VERSION
-        );
-#endif
 
     if (backend == DNN_BACKEND_OPENCV && target == DNN_TARGET_OPENCL_FP16)
         applyTestTag(CV_TEST_TAG_DNN_SKIP_OPENCL_FP16);
-
     testLayerUsingCaffeModels("layer_inner_product", true);
 }
 
@@ -226,6 +209,9 @@ TEST_P(Test_Caffe_layers, Pooling_ave)
 
 TEST_P(Test_Caffe_layers, MVN)
 {
+    if(backend == DNN_BACKEND_CUDA)
+        applyTestTag(CV_TEST_TAG_DNN_SKIP_CUDA); /* MVN is unsupported */
+
     testLayerUsingCaffeModels("layer_mvn");
 }
 
@@ -305,12 +291,10 @@ TEST_P(Test_Caffe_layers, Concat)
                      CV_TEST_TAG_DNN_SKIP_IE_NN_BUILDER, CV_TEST_TAG_DNN_SKIP_IE_VERSION);
 #endif
 
-#if INF_ENGINE_VER_MAJOR_LT(2021040000)
     if (backend == DNN_BACKEND_INFERENCE_ENGINE_NGRAPH &&
         (target == DNN_TARGET_OPENCL || target == DNN_TARGET_OPENCL_FP16))
         applyTestTag(target == DNN_TARGET_OPENCL ? CV_TEST_TAG_DNN_SKIP_IE_OPENCL : CV_TEST_TAG_DNN_SKIP_IE_OPENCL_FP16,
                      CV_TEST_TAG_DNN_SKIP_IE_NGRAPH, CV_TEST_TAG_DNN_SKIP_IE_VERSION);
-#endif
 
 #endif
     testLayerUsingCaffeModels("layer_concat");
@@ -385,7 +369,7 @@ TEST_P(Test_Caffe_layers, layer_prelu_fc)
     // Reference output values are in range [-0.0001, 10.3906]
     double l1 = (target == DNN_TARGET_MYRIAD) ? 0.005 : 0.0;
     double lInf = (target == DNN_TARGET_MYRIAD) ? 0.021 : 0.0;
-#if defined(INF_ENGINE_RELEASE) && INF_ENGINE_VER_MAJOR_GE(2020040000)
+#if defined(INF_ENGINE_RELEASE) && INF_ENGINE_VER_MAJOR_EQ(2020040000)
     if (backend == DNN_BACKEND_INFERENCE_ENGINE_NGRAPH && target == DNN_TARGET_OPENCL)
     {
         l1 = 0.006f; lInf = 0.05f;
@@ -439,14 +423,20 @@ TEST_P(Test_Caffe_layers, Conv_Elu)
     net.setPreferableTarget(target);
     Mat out = net.forward();
 
-    normAssert(ref, out, "", default_l1, default_lInf);
+    double l1 = default_l1, lInf = default_lInf;
+    if (target == DNN_TARGET_CUDA_FP16)
+    {
+        l1 = 0.0002;
+        lInf = 0.0005;
+    }
+    normAssert(ref, out, "", l1, lInf);
 }
 
 class Layer_LSTM_Test : public ::testing::Test
 {
 public:
     int numInp, numOut;
-    Mat Wh, Wx, b, h, c;
+    Mat Wh, Wx, b;
     Ptr<LSTMLayer> layer;
     std::vector<Mat> inputs, outputs;
 
@@ -461,17 +451,12 @@ public:
         Wh = Mat::ones(4 * numOut, numOut, CV_32F);
         Wx = Mat::ones(4 * numOut, numInp, CV_32F);
         b  = Mat::ones(4 * numOut, 1, CV_32F);
-        h  = Mat::ones(4, numOut, CV_32F);
-        c  = Mat::ones(4, numOut, CV_32F);
 
         LayerParams lp;
-        lp.blobs.resize(5);
+        lp.blobs.resize(3);
         lp.blobs[0] = Wh;
         lp.blobs[1] = Wx;
         lp.blobs[2] = b;
-        lp.blobs[3] = h;
-        lp.blobs[4] = c;
-
         lp.set<bool>("produce_cell_output", produceCellOutput);
         lp.set<bool>("use_timestamp_dim", useTimestampDim);
 
@@ -519,12 +504,10 @@ TEST_F(Layer_LSTM_Test, get_set_test)
 TEST(Layer_LSTM_Test_Accuracy_with_, CaffeRecurrent)
 {
     LayerParams lp;
-    lp.blobs.resize(5);
+    lp.blobs.resize(3);
     lp.blobs[0] = blobFromNPY(_tf("lstm.prototxt.w_2.npy"));  // Wh
     lp.blobs[1] = blobFromNPY(_tf("lstm.prototxt.w_0.npy"));  // Wx
     lp.blobs[2] = blobFromNPY(_tf("lstm.prototxt.w_1.npy"));  // bias
-    lp.blobs[3] = Mat::zeros(2, 17, CV_32F);                     // h_0
-    lp.blobs[4] = Mat::zeros(2, 17, CV_32F);                     // c_0
     Ptr<LSTMLayer> layer = LSTMLayer::create(lp);
 
     Mat inp = blobFromNPY(_tf("recurrent.input.npy"));
@@ -535,68 +518,6 @@ TEST(Layer_LSTM_Test_Accuracy_with_, CaffeRecurrent)
     normAssert(h_t_reference, outputs[0]);
 }
 
-TEST(Layer_LSTM_Test_Accuracy_with_, HiddenParams)
-{
-    Mat Wx = blobFromNPY(_tf("lstm.hidden.W.npy"));
-    Mat Wh = blobFromNPY(_tf("lstm.hidden.R.npy"));
-    Mat b = blobFromNPY(_tf("lstm.hidden.B.npy"));
-    Mat h0 = blobFromNPY(_tf("lstm.hidden.h0.npy"));
-    Mat c0 = blobFromNPY(_tf("lstm.hidden.c0.npy"));
-
-    const int numHidden = 3;
-    const int numDirs = Wx.size[0];
-    const int numFeatures = Wx.size[2];
-
-    b = b.reshape(1, b.size[0]);
-    Mat bx = b.colRange(0, b.cols / 2);
-    Mat bh = b.colRange(b.cols / 2, b.cols);
-    b = bx + bh;
-
-    // IFGO->IGFO
-    for (int k = 0; k < numDirs; ++k)
-    {
-        float* WxData = Wx.ptr<float>(k);
-        float* WhData = Wh.ptr<float>(k);
-        float* biasData = b.ptr<float>(k);
-        for (int j = 0; j < numHidden; ++j)
-        {
-            for (int i = 0; i < numFeatures; ++i)
-            {
-                std::swap(WxData[(numHidden + j) * numFeatures + i],
-                          WxData[(numHidden * 2 + j) * numFeatures + i]);
-            }
-            for (int i = 0; i < numHidden; ++i)
-            {
-                std::swap(WhData[(numHidden + j) * numHidden + i],
-                          WhData[(numHidden * 2 + j) * numHidden + i]);
-            }
-            std::swap(biasData[numHidden + j], biasData[numHidden * 2 + j]);
-        }
-    }
-
-    Wx = Wx.reshape(1, Wx.size[0] * Wx.size[1]);
-    Wh = Wh.reshape(1, Wh.size[0] * Wh.size[1]);
-    h0 = h0.reshape(1, h0.size[0] * h0.size[1]);
-    c0 = c0.reshape(1, c0.size[0] * c0.size[1]);
-
-    LayerParams lstmParams;
-    lstmParams.blobs.resize(5);
-    lstmParams.blobs[0] = Wh;
-    lstmParams.blobs[1] = Wx;
-    lstmParams.blobs[2] = b;
-    lstmParams.blobs[3] = h0;
-    lstmParams.blobs[4] = c0;
-    lstmParams.set("bidirectional", false);
-    Ptr<LSTMLayer> layer = LSTMLayer::create(lstmParams);
-
-    Mat inp = blobFromNPY(_tf("lstm.hidden.input.npy"));
-    std::vector<Mat> inputs(1, inp), outputs;
-    runLayer(layer, inputs, outputs);
-
-    Mat h_t_reference = blobFromNPY(_tf("lstm.hidden.output.npy"));
-    normAssert(h_t_reference, outputs[0]);
-}
-
 TEST(Layer_RNN_Test_Accuracy_with_, CaffeRecurrent)
 {
     Ptr<RNNLayer> layer = RNNLayer::create(LayerParams());
@@ -641,9 +562,6 @@ TEST(Layer_LSTM_Test_Accuracy_, Reverse)
     bias.at<float>(2, 0) = 1e10f;  // Output gate - always output everything
     bias.at<float>(3, 0) = 0.f;  // Update signal
 
-    cv::Mat hInternal = cv::Mat::zeros(1, 1, CV_32FC1);
-    cv::Mat cInternal = cv::Mat::zeros(1, 1, CV_32FC1);
-
     LayerParams lp;
     lp.set("reverse", true);
     lp.set("use_timestamp_dim", true);
@@ -651,8 +569,6 @@ TEST(Layer_LSTM_Test_Accuracy_, Reverse)
     lp.blobs.push_back(Wh);
     lp.blobs.push_back(Wx);
     lp.blobs.push_back(bias);
-    lp.blobs.push_back(hInternal);
-    lp.blobs.push_back(cInternal);
 
     cv::Ptr<cv::dnn::LSTMLayer> layer = LSTMLayer::create(lp);
     std::vector<cv::Mat> outputs;
@@ -741,8 +657,6 @@ TEST_P(Test_Caffe_layers, DataAugmentation)
     if (backend == DNN_BACKEND_OPENCV && target == DNN_TARGET_OPENCL_FP16)
         applyTestTag(CV_TEST_TAG_DNN_SKIP_OPENCL_FP16);
     testLayerUsingCaffeModels("data_augmentation", true, false);
-    testLayerUsingCaffeModels("data_augmentation_2x1", true, false);
-    testLayerUsingCaffeModels("data_augmentation_8x6", true, false);
 }
 
 TEST_P(Test_Caffe_layers, Resample)
@@ -787,6 +701,11 @@ TEST_P(Test_Caffe_layers, ROIPooling_Accuracy)
 
     double l1 = (target == DNN_TARGET_OPENCL_FP16 || target == DNN_TARGET_MYRIAD) ? 1e-3 : 1e-5;
     double lInf = (target == DNN_TARGET_OPENCL_FP16 || target == DNN_TARGET_MYRIAD) ? 1e-3 : 1e-4;
+    if (target == DNN_TARGET_CUDA_FP16)
+    {
+        l1 = 2e-4;
+        lInf = 9e-4;
+    }
     normAssert(out, ref, "", l1, lInf);
 }
 
@@ -798,6 +717,8 @@ TEST_P(Test_Caffe_layers, FasterRCNN_Proposal)
         applyTestTag(CV_TEST_TAG_DNN_SKIP_IE_NN_BUILDER);
     if (backend == DNN_BACKEND_INFERENCE_ENGINE_NGRAPH)
         applyTestTag(CV_TEST_TAG_DNN_SKIP_IE_NGRAPH);
+    if(backend == DNN_BACKEND_CUDA)
+        applyTestTag(CV_TEST_TAG_DNN_SKIP_CUDA); /* Proposal layer is unsupported */
 
     Net net = readNetFromCaffe(_tf("net_faster_rcnn_proposal.prototxt"));
 
@@ -1036,6 +957,11 @@ TEST_P(Test_Caffe_layers, PriorBox_repeated)
 
     double l1 = (target == DNN_TARGET_OPENCL_FP16 || target == DNN_TARGET_MYRIAD) ? 1e-3 : 1e-5;
     double lInf = (target == DNN_TARGET_OPENCL_FP16 || target == DNN_TARGET_MYRIAD) ? 1e-3 : 1e-4;
+    if (target == DNN_TARGET_CUDA_FP16)
+    {
+        l1 = 7e-5;
+        lInf = 0.0005;
+    }
     normAssert(out, ref, "", l1, lInf);
 }
 
@@ -1071,7 +997,9 @@ TEST_P(Test_Caffe_layers, PriorBox_squares)
                                        0.25, 0.0, 1.0, 1.0,
                                        0.1f, 0.1f, 0.2f, 0.2f,
                                        0.1f, 0.1f, 0.2f, 0.2f);
-    double l1 = (target == DNN_TARGET_OPENCL_FP16 || target == DNN_TARGET_MYRIAD) ? 2e-5 : 1e-5;
+    double l1 = 1e-5;
+    if (target == DNN_TARGET_OPENCL_FP16 || target == DNN_TARGET_MYRIAD || target == DNN_TARGET_CUDA_FP16)
+        l1 = 2e-5;
     normAssert(out.reshape(1, 4), ref, "", l1);
 }
 
@@ -1205,16 +1133,19 @@ TEST_P(Layer_Test_Convolution_DLDT, Accuracy)
     const Backend backendId = get<0>(GetParam());
     const Target targetId = get<1>(GetParam());
 
-    if (backendId == DNN_BACKEND_INFERENCE_ENGINE_NN_BUILDER_2019 && targetId == DNN_TARGET_MYRIAD)
-        applyTestTag(CV_TEST_TAG_DNN_SKIP_IE_MYRIAD, CV_TEST_TAG_DNN_SKIP_IE_NN_BUILDER);
-
     if (backendId != DNN_BACKEND_INFERENCE_ENGINE_NN_BUILDER_2019 && backendId != DNN_BACKEND_INFERENCE_ENGINE_NGRAPH)
         throw SkipTestException("No support for async forward");
 
-    ASSERT_EQ(DNN_BACKEND_INFERENCE_ENGINE_NGRAPH, backendId);
+    if (backendId == DNN_BACKEND_INFERENCE_ENGINE_NN_BUILDER_2019)
+        setInferenceEngineBackendType(CV_DNN_BACKEND_INFERENCE_ENGINE_NN_BUILDER_API);
+    else if (backendId == DNN_BACKEND_INFERENCE_ENGINE_NGRAPH)
+        setInferenceEngineBackendType(CV_DNN_BACKEND_INFERENCE_ENGINE_NGRAPH);
+    else
+        FAIL() << "Unknown backendId";
 
+    std::string suffix = (targetId == DNN_TARGET_OPENCL_FP16 || targetId == DNN_TARGET_MYRIAD) ? "_fp16" : "";
     Net netDefault = readNet(_tf("layer_convolution.caffemodel"), _tf("layer_convolution.prototxt"));
-    Net net = readNet(_tf("layer_convolution.xml"), _tf("layer_convolution.bin"));
+    Net net = readNet(_tf("layer_convolution" + suffix + ".xml"), _tf("layer_convolution" + suffix + ".bin"));
 
     Mat inp = blobFromNPY(_tf("blob.npy"));
 
@@ -1234,10 +1165,7 @@ TEST_P(Layer_Test_Convolution_DLDT, Accuracy)
 
     std::vector<int> outLayers = net.getUnconnectedOutLayers();
     ASSERT_EQ(net.getLayer(outLayers[0])->name, "output");
-    if (backendId == DNN_BACKEND_INFERENCE_ENGINE_NN_BUILDER_2019)
-        ASSERT_EQ(net.getLayer(outLayers[0])->type, "Convolution");
-    else
-        ASSERT_EQ(net.getLayer(outLayers[0])->type, "Add");
+    ASSERT_EQ(net.getLayer(outLayers[0])->type, "Convolution");
 }
 
 TEST_P(Layer_Test_Convolution_DLDT, setInput_uint8)
@@ -1245,13 +1173,15 @@ TEST_P(Layer_Test_Convolution_DLDT, setInput_uint8)
     const Backend backendId = get<0>(GetParam());
     const Target targetId = get<1>(GetParam());
 
-    if (backendId == DNN_BACKEND_INFERENCE_ENGINE_NN_BUILDER_2019 && targetId == DNN_TARGET_MYRIAD)
-        applyTestTag(CV_TEST_TAG_DNN_SKIP_IE_MYRIAD, CV_TEST_TAG_DNN_SKIP_IE_NN_BUILDER);
-
     if (backendId != DNN_BACKEND_INFERENCE_ENGINE_NN_BUILDER_2019 && backendId != DNN_BACKEND_INFERENCE_ENGINE_NGRAPH)
         throw SkipTestException("No support for async forward");
 
-    ASSERT_EQ(DNN_BACKEND_INFERENCE_ENGINE_NGRAPH, backendId);
+    if (backendId == DNN_BACKEND_INFERENCE_ENGINE_NN_BUILDER_2019)
+        setInferenceEngineBackendType(CV_DNN_BACKEND_INFERENCE_ENGINE_NN_BUILDER_API);
+    else if (backendId == DNN_BACKEND_INFERENCE_ENGINE_NGRAPH)
+        setInferenceEngineBackendType(CV_DNN_BACKEND_INFERENCE_ENGINE_NGRAPH);
+    else
+        FAIL() << "Unknown backendId";
 
     int blobSize[] = {2, 6, 75, 113};
     Mat inputs[] = {Mat(4, &blobSize[0], CV_8U), Mat()};
@@ -1259,10 +1189,12 @@ TEST_P(Layer_Test_Convolution_DLDT, setInput_uint8)
     randu(inputs[0], 0, 255);
     inputs[0].convertTo(inputs[1], CV_32F);
 
+    std::string suffix = (targetId == DNN_TARGET_OPENCL_FP16 || targetId == DNN_TARGET_MYRIAD) ? "_fp16" : "";
+
     Mat outs[2];
     for (int i = 0; i < 2; ++i)
     {
-        Net net = readNet(_tf("layer_convolution.xml"), _tf("layer_convolution.bin"));
+        Net net = readNet(_tf("layer_convolution" + suffix + ".xml"), _tf("layer_convolution" + suffix + ".bin"));
         net.setPreferableBackend(backendId);
         net.setPreferableTarget(targetId);
         net.setInput(inputs[i]);
@@ -1278,16 +1210,19 @@ TEST_P(Layer_Test_Convolution_DLDT, multithreading)
     const Backend backendId = get<0>(GetParam());
     const Target targetId = get<1>(GetParam());
 
-    if (backendId == DNN_BACKEND_INFERENCE_ENGINE_NN_BUILDER_2019 && targetId == DNN_TARGET_MYRIAD)
-        applyTestTag(CV_TEST_TAG_DNN_SKIP_IE_MYRIAD, CV_TEST_TAG_DNN_SKIP_IE_NN_BUILDER);
-
     if (backendId != DNN_BACKEND_INFERENCE_ENGINE_NN_BUILDER_2019 && backendId != DNN_BACKEND_INFERENCE_ENGINE_NGRAPH)
         throw SkipTestException("No support for async forward");
 
-    ASSERT_EQ(DNN_BACKEND_INFERENCE_ENGINE_NGRAPH, backendId);
+    if (backendId == DNN_BACKEND_INFERENCE_ENGINE_NN_BUILDER_2019)
+        setInferenceEngineBackendType(CV_DNN_BACKEND_INFERENCE_ENGINE_NN_BUILDER_API);
+    else if (backendId == DNN_BACKEND_INFERENCE_ENGINE_NGRAPH)
+        setInferenceEngineBackendType(CV_DNN_BACKEND_INFERENCE_ENGINE_NGRAPH);
+    else
+        FAIL() << "Unknown backendId";
 
-    std::string xmlPath = _tf("layer_convolution.xml");
-    std::string binPath = _tf("layer_convolution.bin");
+    std::string suffix = (targetId == DNN_TARGET_OPENCL_FP16 || targetId == DNN_TARGET_MYRIAD) ? "_fp16" : "";
+    std::string xmlPath = _tf("layer_convolution" + suffix + ".xml");
+    std::string binPath = _tf("layer_convolution" + suffix + ".bin");
     Net firstNet = readNet(xmlPath, binPath);
     Net secondNet = readNet(xmlPath, binPath);
     Mat inp = blobFromNPY(_tf("blob.npy"));
@@ -1346,7 +1281,8 @@ TEST_P(Test_DLDT_two_inputs_3dim, as_IR)
     int secondInpType = get<1>(GetParam());
     Target targetId = get<2>(GetParam());
 
-    Net net = readNet(_tf("net_two_inputs.xml"), _tf("net_two_inputs.bin"));
+    std::string suffix = (targetId == DNN_TARGET_OPENCL_FP16 || targetId == DNN_TARGET_MYRIAD) ? "_fp16" : "";
+    Net net = readNet(_tf("net_two_inputs" + suffix + ".xml"), _tf("net_two_inputs.bin"));
     std::vector<int> inpSize = get<3>(GetParam());
     Mat firstInp(3, inpSize.data(), firstInpType);
     Mat secondInp(3, inpSize.data(), secondInpType);
@@ -1377,6 +1313,54 @@ INSTANTIATE_TEST_CASE_P(/*nothing*/, Test_DLDT_two_inputs_3dim, Combine(
   testing::ValuesIn(list_sizes)
 ));
 
+typedef testing::TestWithParam<tuple<int, int, tuple<Backend, Target> > > Test_DLDT_two_inputs;
+TEST_P(Test_DLDT_two_inputs, as_backend)
+{
+    static const float kScale = 0.5f;
+    static const float kScaleInv = 1.0f / kScale;
+
+    Backend backendId = get<0>(get<2>(GetParam()));
+    Target targetId = get<1>(get<2>(GetParam()));
+
+    Net net;
+    LayerParams lp;
+    lp.type = "Eltwise";
+    lp.name = "testLayer";
+    lp.set("operation", "sum");
+    int eltwiseId = net.addLayerToPrev(lp.name, lp.type, lp);  // connect to a first input
+    net.connect(0, 1, eltwiseId, 1);  // connect to a second input
+
+    int inpSize[] = {1, 2, 3, 4};
+    Mat firstInp(4, &inpSize[0], get<0>(GetParam()));
+    Mat secondInp(4, &inpSize[0], get<1>(GetParam()));
+    randu(firstInp, 0, 255);
+    randu(secondInp, 0, 255);
+
+    net.setInputsNames({"data", "second_input"});
+    net.setInput(firstInp, "data", kScale);
+    net.setInput(secondInp, "second_input", kScaleInv);
+    net.setPreferableBackend(backendId);
+    net.setPreferableTarget(targetId);
+    Mat out = net.forward();
+
+    Mat ref;
+    addWeighted(firstInp, kScale, secondInp, kScaleInv, 0, ref, CV_32F);
+    // Output values are in range [0, 637.5].
+    double l1 = (targetId == DNN_TARGET_OPENCL_FP16 || targetId == DNN_TARGET_MYRIAD) ? 0.06 : 1e-6;
+    double lInf = (targetId == DNN_TARGET_OPENCL_FP16 || targetId == DNN_TARGET_MYRIAD) ? 0.3 : 1e-5;
+    if (targetId == DNN_TARGET_CUDA_FP16)
+    {
+        l1 = 0.06;
+        lInf = 0.3;
+    }
+    normAssert(out, ref, "", l1, lInf);
+}
+
+INSTANTIATE_TEST_CASE_P(/*nothing*/, Test_DLDT_two_inputs, Combine(
+  Values(CV_8U, CV_32F), Values(CV_8U, CV_32F),
+  dnnBackendsAndTargets()
+));
+
 class UnsupportedLayer : public Layer
 {
 public:
@@ -1431,12 +1415,6 @@ static void test_dldt_fused_output(Backend backend, Target target)
 
 TEST_P(Test_DLDT_layers, fused_output)
 {
-#if defined(INF_ENGINE_RELEASE) && INF_ENGINE_VER_MAJOR_EQ(2022010000)
-    // IE exception: Cannot get memory!
-    if (backend == DNN_BACKEND_INFERENCE_ENGINE_NGRAPH && target == DNN_TARGET_CPU)
-        applyTestTag(CV_TEST_TAG_DNN_SKIP_IE_CPU, CV_TEST_TAG_DNN_SKIP_IE_NGRAPH, CV_TEST_TAG_DNN_SKIP_IE_VERSION);
-#endif
-
     CV_DNN_REGISTER_LAYER_CLASS(Unsupported, UnsupportedLayer);
     try
     {
@@ -1596,20 +1574,6 @@ TEST_P(Test_Caffe_layers, Interp)
 TEST_P(Test_Caffe_layers, DISABLED_Interp)  // requires patched protobuf (available in OpenCV source tree only)
 #endif
 {
-#if defined(INF_ENGINE_RELEASE) && INF_ENGINE_VER_MAJOR_EQ(2022010000)
-    // Cannot get memory!
-    if (backend == DNN_BACKEND_INFERENCE_ENGINE_NGRAPH && target == DNN_TARGET_CPU)
-        applyTestTag(CV_TEST_TAG_DNN_SKIP_IE_CPU, CV_TEST_TAG_DNN_SKIP_IE_NGRAPH, CV_TEST_TAG_DNN_SKIP_IE_VERSION);
-    // Cannot get memory!
-    if (backend == DNN_BACKEND_INFERENCE_ENGINE_NGRAPH && (target == DNN_TARGET_OPENCL || target == DNN_TARGET_OPENCL_FP16))
-        applyTestTag(target == DNN_TARGET_OPENCL ? CV_TEST_TAG_DNN_SKIP_IE_OPENCL : CV_TEST_TAG_DNN_SKIP_IE_OPENCL_FP16,
-            CV_TEST_TAG_DNN_SKIP_IE_NGRAPH, CV_TEST_TAG_DNN_SKIP_IE_VERSION
-        );
-#elif defined(INF_ENGINE_RELEASE) && INF_ENGINE_VER_MAJOR_EQ(2021030000)
-    if (backend == DNN_BACKEND_INFERENCE_ENGINE_NGRAPH && target == DNN_TARGET_MYRIAD)
-        applyTestTag(CV_TEST_TAG_DNN_SKIP_IE_MYRIAD, CV_TEST_TAG_DNN_SKIP_IE_NGRAPH);  // exception
-#endif
-
     if (backend == DNN_BACKEND_INFERENCE_ENGINE_NN_BUILDER_2019 && target == DNN_TARGET_MYRIAD)
         applyTestTag(CV_TEST_TAG_DNN_SKIP_IE_MYRIAD);
 
@@ -1701,8 +1665,17 @@ TEST_P(Layer_Test_ShuffleChannel, Accuracy)
     net.setPreferableTarget(targetId);
     Mat out = net.forward();
 
-    double l1 = (targetId == DNN_TARGET_OPENCL_FP16) ? 5e-2 : 1e-5;
-    double lInf = (targetId == DNN_TARGET_OPENCL_FP16) ? 7e-2 : 1e-4;
+    double l1 = 1e-5, lInf = 1e-4;
+    if (targetId == DNN_TARGET_OPENCL_FP16)
+    {
+        l1 = 5e-2;
+        lInf = 7e-2;
+    }
+    else if (targetId == DNN_TARGET_CUDA_FP16)
+    {
+        l1 = 0.06;
+        lInf = 0.07;
+    }
     for (int n = 0; n < inpShapeVec[0]; ++n)
     {
         for (int c = 0; c < inpShapeVec[1]; ++c)
@@ -1757,6 +1730,9 @@ TEST_P(Layer_Test_Eltwise_unequal, accuracy_input_0_truncate)
     int backendId = get<0>(get<1>(GetParam()));
     int targetId = get<1>(get<1>(GetParam()));
 
+    if (backendId == DNN_BACKEND_CUDA && weighted)
+        applyTestTag(CV_TEST_TAG_DNN_SKIP_CUDA);
+
     Net net;
     LayerParams lp;
     lp.type = "Eltwise";
@@ -1826,6 +1802,9 @@ TEST_P(Layer_Test_Eltwise_unequal, accuracy_input_0)
     lp.name = "testLayer";
     lp.set<std::string>("output_channels_mode", "input_0");
 
+    if (backendId == DNN_BACKEND_CUDA && weighted)
+        applyTestTag(CV_TEST_TAG_DNN_SKIP_CUDA);
+
     const int inpShapes[][4] = {{1, 4, 2, 2}, {1, 2, 2, 2}, {1, 3, 2, 2}};
     const int out_channels = inpShapes[0][1];
     std::vector<String> inpNames(3);
@@ -2028,7 +2007,7 @@ TEST_P(Layer_Test_Slice, variable_input_shape)
     int targetId = get<1>(GetParam());
 
     int begin[] = {0, 0, 0, 0};
-    int end[] = {INT_MAX, INT_MAX, INT_MAX, INT_MAX};
+    int end[] = {-1, -1, -1, -1};
 
     Net net;
     LayerParams lp;
@@ -2112,407 +2091,4 @@ TEST_P(Layer_Test_BatchNorm, fusion)
 
 INSTANTIATE_TEST_CASE_P(/**/, Layer_Test_BatchNorm, dnnBackendsAndTargets());
 
-class TestLayerFusion : public DNNTestLayer {
-public:
-    static void makeDefaultTestConvolutionLayer(LayerParams& convParams, int in_channels, int num_filters, bool bias_term)
-    {
-        const int kernel_h = 3, kernel_w = 3;
-        const int pad_h = kernel_h / 2, pad_w = kernel_w / 2;
-
-        convParams.set("kernel_h", kernel_h);
-        convParams.set("kernel_w", kernel_w);
-        convParams.set("pad_h", pad_h);
-        convParams.set("pad_w", pad_w);
-        convParams.set("num_output", num_filters);
-        convParams.set("bias_term", bias_term);
-        convParams.type = "Convolution";
-        convParams.name = "convolution";
-
-        float conv_init_magnitude = 1.0f / in_channels / kernel_h / kernel_w;
-        int weightsShape[] = {num_filters, in_channels, kernel_h, kernel_w};
-        Mat weights(4, &weightsShape[0], CV_32F);
-        randu(weights, -conv_init_magnitude, conv_init_magnitude);
-        convParams.blobs.push_back(weights);
-        if (bias_term)
-        {
-            Mat bias(1, num_filters, CV_32F);
-            randu(bias, -1.0f, 1.0f);
-            convParams.blobs.push_back(bias);
-        }
-    }
-
-    static void makeDefaultTestActivationLayer(LayerParams& activationParams, const std::string& type, int in_channels)
-    {
-        activationParams.type = type;
-        activationParams.name = "activation";
-        if (activationParams.type == "ReLU")
-            activationParams.set("negative_slope", 0.1f);
-        else if (activationParams.type == "Power")
-        {
-            activationParams.set("power", 2.0f);
-            activationParams.set("scale", 0.5f);
-            activationParams.set("shift", 0.3f);
-        }
-        else if (activationParams.type == "ReLU6")
-        {
-            activationParams.set("min_value", -1.0f);
-            activationParams.set("max_value", 1.0f);
-        }
-        else if (activationParams.type == "ChannelsPReLU")
-        {
-            Mat scales(1, in_channels, CV_32F);
-            randu(scales, -1.0f, 1.0f);
-            activationParams.blobs.push_back(scales);
-        }
-        else if (activationParams.type == "Exp")
-        {
-            activationParams.set("base", -1.0f);
-            activationParams.set("scale", 0.3f);
-            activationParams.set("shift", 0.6f);
-        }
-    }
-
-    static void makeDefaultTestEltwiseLayer(LayerParams& eltwiseParams, const std::string& op, bool withCoefficients)
-    {
-        eltwiseParams.type = "Eltwise";
-        eltwiseParams.name = "eltwise";
-        eltwiseParams.set("operation", op);
-        if (withCoefficients)
-        {
-            float coeff[] = {0.3f, 0.5f};
-            eltwiseParams.set("coeff", DictValue::arrayReal<float*>(coeff, 2));
-        }
-    }
-
-    static void test(Mat& input, Net& net, Backend backendId, Target targetId, std::vector<int> expectedFusedLayers = std::vector<int>(), double l1 = 0.0, double lInf = 0.0)
-    {
-        DNNTestLayer::checkBackend(backendId, targetId);
-
-        net.enableFusion(false);
-        net.setPreferableBackend(DNN_BACKEND_OPENCV);
-        net.setPreferableTarget(DNN_TARGET_CPU);
-        net.setInput(input);
-        Mat outputReference = net.forward().clone();
-        std::vector<double> refTimings;
-        net.getPerfProfile(refTimings);
-        for (int i = 0; i < refTimings.size(); i++)
-        {
-            CV_Assert(refTimings[i] != 0.0);
-        }
-
-        net.enableFusion(true);
-        net.setPreferableBackend(backendId);
-        net.setPreferableTarget(targetId);
-        net.setInput(input);
-        Mat outputTest = net.forward().clone();
-        std::vector<double> testTimings;
-        net.getPerfProfile(testTimings);
-        for (int i = 0; i < testTimings.size(); i++)
-        {
-            if(std::find(expectedFusedLayers.begin(), expectedFusedLayers.end(), i + 1) != expectedFusedLayers.end())
-            {
-                EXPECT_EQ(testTimings[i], 0.0);
-            }
-            else
-            {
-                EXPECT_NE(testTimings[i], 0.0);
-            }
-        }
-
-        // double ref_max_value, ref_min_value;
-        // minMaxLoc(outputReference.reshape(1, 1), &ref_min_value, &ref_max_value);
-        // std::cout << "reference range: " << ref_min_value << ' ' << ref_max_value << std::endl;
-
-        double default_l1, default_lInf;
-        DNNTestLayer::getDefaultThresholds(backendId, targetId, &default_l1, &default_lInf);
-        if (l1 == 0.0)
-            l1 = default_l1;
-        if (lInf == 0.0)
-            lInf = default_lInf;
-        normAssert(outputReference, outputTest, "", l1, lInf);
-    }
-
-    static testing::internal::ParamGenerator<std::string> eltwiseOpList()
-    {
-        // TODO: automate list generation
-        return Values("sum", "max", "prod", "div");
-    }
-
-    static testing::internal::ParamGenerator<std::string> activationLayersList()
-    {
-        // TODO: automate list generation
-        return Values("ReLU", "ReLU6", "ChannelsPReLU", "TanH", "Swish", "Mish", "Sigmoid", "ELU", "AbsVal", "BNLL", "Power", "Exp");
-    }
-
-    static testing::internal::ParamGenerator<tuple<Backend, Target> > dnnBackendsAndTargetsForFusionTests()
-    {
-        return dnnBackendsAndTargets(false, false, true, false); // OCV OpenCL + OCV CPU
-    }
-};
-
-typedef TestWithParam<tuple<bool, std::string, tuple<Backend, Target> > > ConvolutionActivationFusion;
-TEST_P(ConvolutionActivationFusion, Accuracy)
-{
-    //          input
-    //            |
-    // -----------------------
-    // |     convolution     |
-    // -----------------------
-    //            |
-    // -----------------------
-    // |     activation      |
-    // -----------------------
-    //            |
-    //         output
-
-    const int batch_size = 2, in_channels = 16;
-    const int in_height = 16, in_width = 16;
-    int inputShape[] = {batch_size, in_channels, in_height, in_width};
-    Mat input(4, &inputShape[0], CV_32F);
-    randu(input, 1.0f, 2.0f);
-
-    bool bias_term = get<0>(GetParam());
-    LayerParams convParams;
-    TestLayerFusion::makeDefaultTestConvolutionLayer(convParams, in_channels, in_channels, bias_term);
-
-    std::string actType = get<1>(GetParam());
-    LayerParams activationParams;
-    TestLayerFusion::makeDefaultTestActivationLayer(activationParams, actType, in_channels);
-
-    Backend backendId = get<0>(get<2>(GetParam()));
-    Target targetId = get<1>(get<2>(GetParam()));
-
-    Net net;
-    int convId = net.addLayer(convParams.name, convParams.type, convParams);
-    int activId = net.addLayerToPrev(activationParams.name, activationParams.type, activationParams);
-    net.connect(0, 0, convId, 0);
-
-    std::vector<int> expectedFusedLayers;
-    if (backendId == DNN_BACKEND_OPENCV)
-    {
-        if (targetId == DNN_TARGET_CPU)
-            expectedFusedLayers.push_back(activId); // all activations are fused
-        else if (targetId == DNN_TARGET_OPENCL || targetId == DNN_TARGET_OPENCL_FP16)
-        {
-            if (actType == "ReLU" || actType == "ChannelsPReLU" || actType == "ReLU6" || actType == "TanH" /*|| actType == "Power"*/)
-                expectedFusedLayers.push_back(activId);
-        }
-    }
-
-    TestLayerFusion::test(input, net, backendId, targetId, expectedFusedLayers);
-}
-INSTANTIATE_TEST_CASE_P(TestLayerFusion, ConvolutionActivationFusion, Combine(
-/* bias */       testing::Bool(),
-/* activation */ TestLayerFusion::activationLayersList(),
-                 TestLayerFusion::dnnBackendsAndTargetsForFusionTests()
-));
-
-typedef TestWithParam<tuple<bool, std::string, bool, tuple<Backend, Target> > > ConvolutionEltwiseFusion;
-TEST_P(ConvolutionEltwiseFusion, Accuracy)
-{
-    //                 input
-    //                   |
-    //    -------------------------------
-    //    |                             |
-    //    |                      ---------------
-    //    |                      | convolution |
-    //    |                      ---------------
-    //    |                             |
-    //    |       ----------------      |
-    //    --------|  eltwise op  |-------
-    //            ----------------
-    //                   |
-    //                 output
-
-    const int batch_size = 2, in_channels = 16;
-    const int in_height = 16, in_width = 16;
-    int inputShape[] = {batch_size, in_channels, in_height, in_width};
-    Mat input(4, &inputShape[0], CV_32F);
-    randu(input, 1.0f, 2.0f); // avoid small values to test eltwise div
-
-    bool bias_term = get<0>(GetParam());
-    LayerParams convParams;
-    TestLayerFusion::makeDefaultTestConvolutionLayer(convParams, in_channels, in_channels, bias_term);
-
-    std::string eltwiseOp = get<1>(GetParam());
-    bool weightedEltwise = get<2>(GetParam());
-    if (eltwiseOp != "sum" && weightedEltwise)
-            throw SkipTestException("weighted eltwise not supported");
-    LayerParams eltwiseParams;
-    TestLayerFusion::makeDefaultTestEltwiseLayer(eltwiseParams, eltwiseOp, weightedEltwise);
-
-    Net net;
-    int convId = net.addLayer(convParams.name, convParams.type, convParams);
-    int eltwiseId = net.addLayer(eltwiseParams.name, eltwiseParams.type, eltwiseParams);
-    net.connect(0, 0, convId, 0);
-    net.connect(convId, 0, eltwiseId, 0);
-    net.connect(0, 0, eltwiseId, 1);
-
-    Backend backendId = get<0>(get<3>(GetParam()));
-    Target targetId = get<1>(get<3>(GetParam()));
-    TestLayerFusion::test(input, net, backendId, targetId);
-}
-INSTANTIATE_TEST_CASE_P(TestLayerFusion, ConvolutionEltwiseFusion, Combine(
-/* bias */              testing::Bool(),
-/* eltwise op */        TestLayerFusion::eltwiseOpList(),
-/* eltwise weighted */  testing::Bool(),
-                        TestLayerFusion::dnnBackendsAndTargetsForFusionTests()
-));
-
-typedef TestWithParam<tuple<bool, std::string, bool, std::string, tuple<Backend, Target> > > ConvolutionEltwiseActivationFusion;
-TEST_P(ConvolutionEltwiseActivationFusion, Accuracy)
-{
-    //                 input
-    //                   |
-    //    -------------------------------
-    //    |                             |
-    //    |                      ---------------
-    //    |                      | convolution |
-    //    |                      ---------------
-    //    |                             |
-    //    |       ----------------      |
-    //    --------|  eltwise op  |-------
-    //            ----------------
-    //                   |
-    //            ----------------
-    //            |  activation  |
-    //            ----------------
-    //                   |
-    //                output
-
-    const int batch_size = 2, in_channels = 16;
-    const int in_height = 16, in_width = 16;
-    int inputShape[] = {batch_size, in_channels, in_height, in_width};
-    Mat input(4, &inputShape[0], CV_32F);
-    randu(input, 1.0f, 2.0f); // avoid small values to test eltwise div
-
-    bool bias_term = get<0>(GetParam());
-    LayerParams convParams;
-    TestLayerFusion::makeDefaultTestConvolutionLayer(convParams, in_channels, in_channels, bias_term);
-
-    std::string eltwiseOp = get<1>(GetParam());
-    bool weightedEltwise = get<2>(GetParam());
-    if (eltwiseOp != "sum" && weightedEltwise)
-            throw SkipTestException("weighted eltwise not supported");
-    LayerParams eltwiseParams;
-    TestLayerFusion::makeDefaultTestEltwiseLayer(eltwiseParams, eltwiseOp, weightedEltwise);
-
-    std::string actType = get<3>(GetParam());
-    LayerParams activationParams;
-    TestLayerFusion::makeDefaultTestActivationLayer(activationParams, actType, in_channels);
-
-    Backend backendId = get<0>(get<4>(GetParam()));
-    Target targetId = get<1>(get<4>(GetParam()));
-
-    Net net;
-    int convId = net.addLayer(convParams.name, convParams.type, convParams);
-    int eltwiseId = net.addLayer(eltwiseParams.name, eltwiseParams.type, eltwiseParams);
-    int activId = net.addLayer(activationParams.name, activationParams.type, activationParams);
-    net.connect(0, 0, convId, 0);
-    net.connect(convId, 0, eltwiseId, 0);
-    net.connect(0, 0, eltwiseId, 1);
-    net.connect(eltwiseId, 0, activId, 0);
-
-    std::vector<int> expectedFusedLayers;
-    if (backendId == DNN_BACKEND_OPENCV)
-    {
-        if (targetId == DNN_TARGET_CPU)
-            expectedFusedLayers.push_back(activId); // activation is fused with eltwise layer
-        else if (targetId == DNN_TARGET_OPENCL || targetId == DNN_TARGET_OPENCL_FP16)
-        {
-            if (eltwiseOp == "sum" && !weightedEltwise &&
-                (actType == "ReLU" || actType == "ChannelsPReLU" /*|| actType == "Power"*/)
-            )
-            {
-                expectedFusedLayers.push_back(eltwiseId);
-                expectedFusedLayers.push_back(activId);
-            }
-        }
-    }
-
-    TestLayerFusion::test(input, net, backendId, targetId, expectedFusedLayers);
-}
-INSTANTIATE_TEST_CASE_P(TestLayerFusion, ConvolutionEltwiseActivationFusion, Combine(
-/* bias */              testing::Bool(),
-/* eltwise op */        TestLayerFusion::eltwiseOpList(),
-/* eltwise weighted */  testing::Bool(),
-/* activation */        TestLayerFusion::activationLayersList(),
-                        TestLayerFusion::dnnBackendsAndTargetsForFusionTests()
-));
-
-typedef TestWithParam<tuple<bool, std::string, std::string, bool, tuple<Backend, Target> > > ConvolutionActivationEltwiseFusion;
-TEST_P(ConvolutionActivationEltwiseFusion, Accuracy)
-{
-    //                 input
-    //                   |
-    //    -------------------------------
-    //    |                             |
-    //    |                     ----------------
-    //    |                     |  convolution |
-    //    |                     ----------------
-    //    |                             |
-    //    |                     ----------------
-    //    |                     |  activation  |
-    //    |                     ----------------
-    //    |                             |
-    //    |       ----------------      |
-    //    --------| eltwise sum  |-------
-    //            ----------------
-    //                   |
-
-    const int batch_size = 2, in_channels = 16;
-    const int in_height = 16, in_width = 16;
-    int inputShape[] = {batch_size, in_channels, in_height, in_width};
-    Mat input(4, &inputShape[0], CV_32F);
-    randu(input, 1.0f, 2.0f); // avoid small values to test eltwise div
-
-    bool bias_term = get<0>(GetParam());
-    LayerParams convParams;
-    TestLayerFusion::makeDefaultTestConvolutionLayer(convParams, in_channels, in_channels, bias_term);
-
-    std::string actType = get<1>(GetParam());
-    LayerParams activationParams;
-    TestLayerFusion::makeDefaultTestActivationLayer(activationParams, actType, in_channels);
-
-    std::string eltwiseOp = get<2>(GetParam());
-    bool weightedEltwise = get<3>(GetParam());
-    if (eltwiseOp != "sum" && weightedEltwise)
-            throw SkipTestException("weighted eltwise not supported");
-    LayerParams eltwiseParams;
-    TestLayerFusion::makeDefaultTestEltwiseLayer(eltwiseParams, eltwiseOp, weightedEltwise);
-
-    Backend backendId = get<0>(get<4>(GetParam()));
-    Target targetId = get<1>(get<4>(GetParam()));
-
-    Net net;
-    int convId = net.addLayer(convParams.name, convParams.type, convParams);
-    int activId = net.addLayer(activationParams.name, activationParams.type, activationParams);
-    int eltwiseId = net.addLayer(eltwiseParams.name, eltwiseParams.type, eltwiseParams);
-    net.connect(0, 0, convId, 0);
-    net.connect(convId, 0, activId, 0);
-    net.connect(activId, 0, eltwiseId, 0);
-    net.connect(0, 0, eltwiseId, 1);
-
-    std::vector<int> expectedFusedLayers;
-    if (backendId == DNN_BACKEND_OPENCV)
-    {
-        if (targetId == DNN_TARGET_CPU)
-            expectedFusedLayers.push_back(activId); // activation fused with convolution
-        else if (targetId == DNN_TARGET_OPENCL || targetId == DNN_TARGET_OPENCL_FP16)
-        {
-            if (actType == "ReLU" || actType == "ChannelsPReLU" || actType == "ReLU6" || actType == "TanH" /*|| actType == "Power"*/)
-                expectedFusedLayers.push_back(activId); // activation fused with convolution
-        }
-    }
-
-    TestLayerFusion::test(input, net, backendId, targetId, expectedFusedLayers);
-}
-INSTANTIATE_TEST_CASE_P(TestLayerFusion, ConvolutionActivationEltwiseFusion, Combine(
-/* bias */              testing::Bool(),
-/* activation */        TestLayerFusion::activationLayersList(),
-/* eltwise op */        TestLayerFusion::eltwiseOpList(),
-/* eltwise weighted */  testing::Bool(),
-                        TestLayerFusion::dnnBackendsAndTargetsForFusionTests()
-));
-
 }} // namespace
